<?php
require_once "functions.general.php";
require_once "functions.parsing.php";
require_once "talents.php";
require_once "talentIcons.php";
require_once "simpleparser.php";

if (!defined('ROSTER_INSTALLED')) {
	exit('Detected invalid access to this file!');
}

function synchGuild($synchPlayers = true, $realm = null, $guildName = null) {
	global $wowdb, $roster_conf, $addon_conf;
	
	if (empty($realm)) {
		$realm = $roster_conf['server_name'];
	}
	
	if (empty($guildName)) {
		$guildName = $roster_conf['guild_name'];
	}
	
	if (($rosterData = getGuildInfo($realm, $guildName)) && $addon_conf['ArmorySync']['UpdateRoster']) {
		$guild = $rosterData[$realm]['Guild'][$guildName];
		$currentTimestamp = time();
		$currentTime = getDate($currentTimestamp);
		
		// Update the guild
		$guildId = $wowdb->update_guild($realm, $guildName, $currentTime, $guild);
		$guildMembers = $guild['Members'];
	
		// update the list of guild members
		$guildMessages = $wowdb->getMessages();
		foreach(array_keys($guildMembers) as $char_name) {
			$char = $guildMembers[$char_name];
			$wowdb->update_guild_member($guildId, $char_name, $char, $currentTimestamp, array());
			$guildMessages .= $wowdb->getMessages();
			$wowdb->resetMessages();
		}
		
		// Remove the members who were not in this list
		$wowdb->remove_guild_members($guildId, $currentTime);
		$wowdb->remove_guild_members_id($guildId);
		$guildMessages .= $wowdb->getMessages();
		$wowdb->resetMessages();
	
		displayGuildMessages($guildMessages, $wowdb->membersupdated, $wowdb->membersadded, $wowdb->membersremoved, $guildName);
		if ($synchPlayers) {
			synchPlayers(null, array_keys($guildMembers));
		}
	}
}

function synchPlayer($realm = null, $charName, $displayMessages = true) {
	global $wowdb, $roster_conf, $addon_conf;
	
	if (empty($realm)) {
		$realm = $roster_conf['server_name'];
	}
	
	if (empty($charName)) {
		//Throw Error
		print "CRITICAL ERROR: No Player Given.<br>\n";
		return;
	}
	
	$startTime = time();
	if (($rosterData = getPlayerInfo($realm, $charName)) && $addon_conf['ArmorySync']['UpdateRoster']) {
		$guildInfo = $wowdb->get_guild_info($roster_conf['server_name'], $roster_conf['guild_name']);
			
		$wowdb->update_char($guildInfo['guild_id'], $charName, $rosterData);
		
		if ($displayMessages) {
			displayMessages($wowdb->getMessages());
			displayErrors($wowdb->getErrors());
		}
	}
}

function synchMemberByID($realm = null, $memberID, $displayMessages = true) {
	global $wowdb, $roster_conf;

	//Print 'Entered [synchMemberByID] with memberID (' . $memberID . ')passed in';

	if (empty($realm)) {
		$realm = $roster_conf['server_name'];
	}

	if (empty($memberID)) {
		//Throw Error
		print "CRITICAL ERROR: No Player Given.<br>\n";
		return;
	}

	$startTime = time();

	//Get player name from member id

	$query =
		"SELECT name " .
		"FROM `".ROSTER_MEMBERSTABLE."` members WHERE member_id=" .  $memberID;

	$result = $wowdb->query($query)
		or die_quietly($wowdb->error(), 'Database Error', basename(__FILE__), __LINE__, $query);


	$membername = '';
	while ($row = $wowdb->fetch_assoc($result)) {
		$membername = $row["name"];
	}

	if ($membername == ''){
		print 'Failed to get member name!';
	}
	else {
		//Ok process this player, then output code to redirect for next run
		Print 'Scanning Player: ' . $membername;
		if ($rosterData = getPlayerInfo($realm, $membername)) {
			$guildInfo = $wowdb->get_guild_info($roster_conf['server_name'], $roster_conf['guild_name']);
			$wowdb->update_char($guildInfo['guild_id'], $membername, $rosterData);

			if ($displayMessages) {
				displayMessages($wowdb->getMessages());
				displayErrors($wowdb->getErrors());
			}
		}

		//Now get the next player, if one exists...
		
		//Get total players to synch...
		$query =
			"SELECT COUNT(member_id)  totalleft " .
			"FROM `".ROSTER_MEMBERSTABLE."` members" ;
		$wowdb->query("set sql_mode = ''");

		$result = $wowdb->query($query)
			or die_quietly($wowdb->error(), 'Database Error', basename(__FILE__), __LINE__, $query);

		$memberstotal = -1;
		while ($row = $wowdb->fetch_assoc($result)) {
			$memberstotal= $row["totalleft"];
		}


		//Get total players left...
		$query =
			"SELECT COUNT(member_id)  totalleft " .
			"FROM `".ROSTER_MEMBERSTABLE."` members where member_id > $memberID" ;
		$wowdb->query("set sql_mode = ''");

		$result = $wowdb->query($query)
			or die_quietly($wowdb->error(), 'Database Error', basename(__FILE__), __LINE__, $query);

		$membersleft = -1;
		while ($row = $wowdb->fetch_assoc($result)) {
			$membersleft= $memberstotal - $row["totalleft"];
		}

		$query =
			"SELECT min(members.member_id) ID " .
			"FROM `".ROSTER_MEMBERSTABLE."` members where member_id > $memberID" ;

		$wowdb->query("set sql_mode = ''");

		$result = $wowdb->query($query)
			or die_quietly($wowdb->error(), 'Database Error', basename(__FILE__), __LINE__, $query);

		$nextmemberid = -1;
		while ($row = $wowdb->fetch_assoc($result)) {
			$nextmemberid = $row["ID"];
		}

		
		if ($nextmemberid == -1){
			print 'Finished Processing Members';
		}
		else {
			//Get next member name...
			$query =
				"SELECT members.name " .
				"FROM `".ROSTER_MEMBERSTABLE."` members where member_id = $nextmemberid" ;
	
			$wowdb->query("set sql_mode = ''");
	
			$result = $wowdb->query($query)
				or die_quietly($wowdb->error(), 'Database Error', basename(__FILE__), __LINE__, $query);
	
			while ($row = $wowdb->fetch_assoc($result)) {
				$nextmembername = $row["name"];
			}

			//Output javascript to move to process next player...
			Print 'Next Player: ' . $nextmembername . '<br>';
			$percdone = round ($membersleft / $memberstotal * 100);
			Print "Progress: $percdone% $membersleft out of $memberstotal";
			//Print '<pre>' . $module_name . '</pre>';
			$link = 'addon.php?roster_addon_name=ArmorySynch&syncmemberid='.$nextmemberid;

			//Link to next player
			Print '<script LANGUAGE="JavaScript">';
			Print 'function nextMember() {';
			Print '	window.location="' . $link . '";';
			Print '}';
			Print "self.setTimeout('nextMember()', 5000);";
			Print '</script>';
		}


	}



}
function synchPlayers($realm, $players) {
	global $wowdb, $roster_conf;
	
	print $wowdb->getMessages();
	$wowdb->resetMessages();
	
	if (empty($realm)) {
		$realm = $roster_conf['server_name'];
	}
	
	if (empty($players) || !is_array($players)) {
		//Throw Error
		print "CRITICAL ERROR: No Player Given or Empty Array.<br>\n";
		return;
	}
	
	$startTime = time();
	foreach($players as $player) {
		$playerStartTime = time();
		
		synchPlayer(null, $player, false);
		
		print $player . " synched: " . (time() - $playerStartTime) . " seconds.  Total runtime: " . (time() - $startTime) . " seconds.<br>\n";
		flush();
	}
	
	displayMessages($wowdb->getMessages());
	displayErrors($wowdb->getErrors());
}

function getGuildInfo($realm, $guildName) {
	global $roster_conf;
	
	if (empty($realm)) {
		//Throw Error
		print "CRITICAL ERROR: No Guild Realm Given.<br>\n";
		return;
	}
	
	if (empty($guildName)) {
		//Throw Error
		print "CRITICAL ERROR: No Guild Name Given.<br>\n";
		return;
	}
	
	$content = getArmoryContent("guild-info.xml", "r=" . urlencode($realm) . "&n=" . urlencode($guildName));
	
	if (empty($content)) {
		//throw Error
		print "CRITICAL ERROR: Empty Content for Guild.<br>\n";
		return;
	}
	
	$xml_parser = new SimpleParser;
	$xml_parser->parse($content);
	$xml = $xml_parser->data->guildInfo->guild;
	
	if (empty($xml)) {
		//throw Error
		print "CRITICAL ERROR: Incomplete Data for Guild.<br>\n";
		return;
	}
	
	$guild = array();
	$players = array();
		
	parseGuild($xml, $guild);
	$guild["Locale"] = $roster_conf['roster_lang'];
	$guild["Server"] = $realm;
	parseMembers($xml->members, $players);
	
	/*
	Unnecessary - thanks Pleeg
	if ($numPages > 1) {
		for ($i = 2; $i <= $numPages; $i++) {
			$content = getArmoryContent("guild-info.xml", "r=" . $realm . "&n=" . $guildName . "&p=" . $i);
			
			if (empty($content) || $dataNotFound) {
				//throw Error
			}
			
			$xml_parser = new SimpleParser;
			$xml_parser->parse($content);
			$xml = $xml_parser->data->guildInfo;
			
			parseMembers($xml->guild->members, $players);
		}
	}
	*/
	
	$rosterData[$realm]["Guild"][$guildName] = $guild;
	$rosterData[$realm]["Guild"][$guildName]["Members"] = $players;
	
	if ($addon_conf['ArmorySync']['DebugLevel'] >= 1) {
		print "GuildInfo:<br><pre>";
		print_r($rosterData);
		print "</pre><br>";
	}
	
	return $rosterData;
}

function getPlayerInfo($realm, $charName) {
	global $roster_conf, $addon_conf;
	
	if (empty($charName)) {
		//Throw Error
		print "CRITICAL ERROR: No Name given for Player.<br>\n";
		return;
	}
	
	$content = getArmoryContent("character-sheet.xml", "r=" . urlencode($realm) . "&n=" . $charName);
	
	if (empty($content)) {
		//throw Error
		print "Non-Critical Error: Empty Content for " . $charName . ".  Player has probably not updated in awhile.<br>\n";
		return;
	}
	
	$xml_parser = new SimpleParser;
	$xml_parser->parse($content);
	$xml = $xml_parser->data->characterInfo; 
	
	if (empty($xml->characterTab)) {
		//throw Error
		print "Non-Critical Error: Incomplete Data for " . $charName . ".  Player has probably not updated in awhile.<br>\n";
		return;
	}
	
	$guild = array(); 
	$player = array();
	$skills = array(); 
	$repuation = array();
	$equipment = array();
	$talents = array();
	
	parseGuild($xml->character, $guild);
	parsePlayer($xml, $player);
	$player["Locale"] = $addon_conf['ArmorySync']['Locale'];
	parseSkills($xml->skillTab->skillCategory, $skills);
	parseReputation($xml->reputationTab->factionCategory, $reputation);
	parseEquipment($xml->characterTab->items->item, $equipment, $player);
	parseTalents($player, $talents);
	
	$rosterData = $player;
	$rosterData["Guild"] = $guild;
	$rosterData["Reputation"] = $reputation;
	$rosterData["Equipment"] = $equipment;
	$rosterData["Skills"] = $skills;
	$rosterData["Talents"] = $talents;
	
	if ($addon_conf['ArmorySync']['DebugLevel'] >= 1) {
		print "Player Info:<br><pre>";
		print_r($rosterData);
		print "</pre><br>";
	}
		
	return $rosterData;
}
?>